﻿// COPYRIGHT 2013, 2014, 2015 by the Open Rails project.
// 
// This file is part of Open Rails.
// 
// Open Rails is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Open Rails is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Open Rails.  If not, see <http://www.gnu.org/licenses/>.


namespace Orts.Parsers.Msts
{
    /// <summary>
    /// Provides an enum for all the binary tokens used
    /// in the header record in the Kuju compressed
    /// binary file format.
    /// </summary>
    public enum TokenID : uint
    {
        //////////////////////////////////////////////////////////////
        /// These were taken from coreids.tok ( in the UTILS folder )
        //////////////////////////////////////////////////////////////
        error = 0,
        comment = 1,
        point,
        vector,
        quat,
        normals,
        normal_idxs,
        points,
        uv_point,
        uv_points,
        colour,
        colours,
        packed_colour,
        image,
        images,
        texture,
        textures,
        light_material,
        light_materials,
        linear_key,
        tcb_key,
        linear_pos,
        tcb_pos,
        slerp_rot,
        tcb_rot,
        controllers,
        anim_node,
        anim_nodes,
        animation,
        animations,
        anim,
        lod_controls,
        lod_control,
        distance_levels_header,
        distance_level_header,
        dlevel_selection,
        distance_levels,
        distance_level,
        sub_objects,
        sub_object,
        sub_object_header,
        geometry_info,
        geometry_nodes,
        geometry_node,
        geometry_node_map,
        cullable_prims,
        vtx_state,
        vtx_states,
        vertex,
        vertex_uvs,
        vertices,
        vertex_set,
        vertex_sets,
        primitives,
        prim_state,
        prim_states,
        prim_state_idx,
        indexed_point_list,
        point_list,
        indexed_line_list,
        indexed_trilist,
        tex_idxs,
        tri,
        vertex_idxs,
        flags,
        matrix,
        matrices,
        hierarchy,
        volumes,
        vol_sphere,
        shape_header,
        shape,
        shader_names,
        shader_name,
        texture_filter_names,
        texture_filter_name,
        sort_vectors,
        uvop_arg_sets,
        uvop_arg_set,
        light_model_cfgs,
        light_model_cfg,
        uv_ops,
        uvop_copy,
        uv_op_share,
        uv_op_copy,
        uv_op_uniformscale,
        uv_op_user_uninformscale,
        uv_op_nonuniformscale,
        uv_op_user_nonuninformscale,
        uv_op_transform,
        uv_op_user_transform,
        uv_op_reflectxy,
        uv_op_reflectmap,
        uv_op_reflectmapfull,
        uv_op_spheremap,
        uv_op_spheremapfull,
        uv_op_specularmap,
        uv_op_embossbump,
        user_uv_args,
        io_dev,
        io_map,
        sguid,
        dlev_cfg_table,
        dlev_cfg,
        subobject_shaders,
        subobject_light_cfgs,
        shape_named_data,
        shape_named_data_header,
        shape_named_geometry,
        shape_geom_ref,
        material_palette,
        blend_config,
        blend_config_header,
        filtermode_cfgs,
        filter_mode_cfg,
        blend_mode_cfgs,
        blend_mode_cfg,
        texture_stage_progs,
        texture_stage_prog,
        blend_mode_cfg_refs,
        shader_cfgs,
        shader_cfg,
        texture_slots,
        texture_slot,
        named_filter_modes,
        named_filter_mode,
        filtermode_cfg_refs,
        filtermode_cfg_ref,
        named_shaders,
        named_shader,
        shader_cfg_refs,
        shader_cfg_ref,

        terrain_desc,
        terrain_desc_flags,
        terrain_desc_size,
        terrain_desc_tiles,
        terrain,
        terrain_errthreshold_scale,
        terrain_alwaysselect_maxdist,
        terrain_samples,
        terrain_nsamples,
        terrain_sample_rotation,
        terrain_sample_floor,
        terrain_sample_scale,
        terrain_sample_size,
        terrain_sample_fbuffer,
        terrain_sample_ybuffer,
        terrain_sample_ebuffer,
        terrain_sample_nbuffer,
        terrain_sample_cbuffer,
        terrain_sample_dbuffer,
        terrain_shaders,
        terrain_shader,
        terrain_texslots,
        terrain_texslot,
        terrain_uvcalcs,
        terrain_uvcalc,
        terrain_patches,
        terrain_patchsets,
        terrain_patchset,
        terrain_patchset_distance,
        terrain_patchset_npatches,
        terrain_patchset_fbuffer,
        terrain_patchset_patches,
        terrain_patchset_patch,
        terrain_transfers,
        terrain_transfer,
        terrain_shapes,
        terrain_shape,
        world,
        world_shader,
        world_anim_shader,
        world_anim_shader_framelen,
        world_anim_shader_frames,
        world_anim_shader_frame,
        world_anim_shader_frame_uvtiles,
        world_anim_shader_frame_uvscroll,
        world_anim_shader_frame_uvstamp,
        world_fog_distance,
        world_fog_day_colour,
        world_fog_night_colour,
        world_sky,
        world_sky_layers,
        world_sky_nlayers_behind_satellites,
        world_sky_layer,
        world_sky_layer_fadein,
        world_sky_layer_fadeout,
        world_sky_layer_top,
        world_sky_layer_top_nfaces,
        world_sky_layer_top_radius,
        world_sky_layer_top_height,
        world_sky_layer_edge,
        world_sky_layer_edge_steps,
        world_sky_layer_edge_step_height,
        world_sky_layer_edge_step_radius,
        world_sky_horizon,
        world_sky_satellites,
        world_sky_satellite,
        world_sky_satellite_low_scale,
        world_sky_satellite_high_scale,
        world_sky_satellite_rise_time,
        world_sky_satellite_dir_rise_colour,
        world_sky_satellite_dir_high_colour,
        world_sky_satellite_dir_set_colour,
        world_sky_satellite_amb_rise_colour,
        world_sky_satellite_amb_high_colour,
        world_sky_satellite_amb_set_colour,
        world_sky_satellite_rise_position,
        world_sky_satellite_set_time,
        world_sky_satellite_fade_time,
        world_sky_satellite_light,
        world_sky_satellite_fog,
        world_water,
        world_water_wave_height,
        world_water_wave_speed,
        world_water_layers,
        world_water_layer,
        world_water_layer_height,
        world_water_layer_sky_reflection,
        world_precipitation,
        world_precipitation_type,
        world_precipitation_type_rain,
        world_precipitation_type_snow,
        world_precipitation_density,
        world_precipitation_radius,
        world_precipitation_speed,
        world_precipitation_near_z_bias,
        world_precipitation_volume_radius,
        world_precipitation_volume_rel_height,
        world_wind,
        world_wind_layers,
        world_wind_layer,
        world_wind_layer_maxheight,
        world_wind_layer_direction,
        world_wind_layer_speed,
        world_wind_layer_turbulencep,
        world_sky_lensflares,
        world_sky_lensflare,
        world_simple_shaders,
        world_simple_shader,
        world_sky_lens_beads,
        world_sky_lens_bead,
        world_glares,
        world_glare,
        world_blit3duvs,
        max_data,
        max_dlev_data,
        max_smoothing_data,
        max_sort_priorities,
        uv_op_diffusemap,
        world_fog_start_distance,
        terrain_water_height_offset,
        terrain_sample_asbuffer = 281,

        /// //////////////////////////////////////////////////////
        // To avoid duplicates, the rest of these offset up 300 from their actual values
        // as stored in the binary files 
        /// //////////////////////////////////////////////////////

        ///////////////////////////////////////////////////////////
        // These ones were made up by reading values in binary world files
        /// //////////////////////////////////////////////////////
        Static = 303,
        TrackObj = 305,
        Forest = 308,
        CollideObject = 311,
        Signal = 317,
        Platform = 360,
        LevelCr = 362,
        Speedpost = 364,
        Hazard=365,



        ///////////////////////////////////////////////////////////
        // These were taken from the loadstr.hdr file - file encapsulation for ffedit compression
        ///////////////////////////////////////////////////////////

        Tr_Worldfile = 375,   // ie in the binary file this has a token value of 75
        Tr_Watermark,
        Tr_DataFile,
        Tr_ConfigFile,
        Tr_GantryFile,
        Tr_SignalFile,
        Tr_PlayerFile,
        Tr_CabViewFile,
        Tr_RouteFile,
        Tr_Activity_File,
        Tr_Activity_Header,
        Tr_SyncCode,
        Tr_SMS,
        Tr_Worldsoundfile,

        // train config file
        TrainCfgs,
        TrainCfg,
        Graphic,
        Default,
        CameraConfigFile,

        Lights,
        FileName,
        FileNames,
        Position,
        Direction,
        MaxVisDistance,
        Quality,
        StaticDetailLevel,
        StoreMatrix,
        AnimShape,

        StaticFlags,

        CollideFlags,
        CollideFunction,

        Sound,

        UiD,

        // Track section config
        TrackSections,
        TrackSection,
        TrackShapes,
        TrackShape,
        TunnelShape,
        RoadShape,
        CrossoverShape,
        ManualJunctionShape,

        SectionSize,
        SectionFlags,
        SectionIdx,
        SectionCurve,
        SectionSkew,
        XoverPts,
        XoverPt,
        JNodePosn,

        TrackNodes,
        TrackNode,
        TrPins,
        TrPin,
        TrVectorNode,
        TrJunctionNode,
        TrSignalNode,
        TrLevelCrossingNode,
        TrTrackCrossingNode,
        TrEndNode,

        TrSignalDirection,
        TrEndLinkFile,
        TrVectorSections,
        TrWorldPosition,

        // Signal track items

        TrSignal,
        TrSignalType,
        TrSignalDirs,
        TrSignalDir,
        TrSignalRDir,
        TrStateSig,

        // Signal config

        SignalFlags,

        SignalGroups,
        SignalGroup,
        SignalGroupEntries,

        LightsTab,
        LightsTabEntry,
        LightTextures,
        LightTex,
        SignalLightTex,

        SignalTypes,
        SignalType,
        SignalFnType,
        SignalSubObjs,
        SignalSubObj,
        SignalShapes,
        SignalShape,
        SignalDirLights,
        SignalDirLight,
        SignalLights,
        SignalLight,
        SignalDrawStates,
        SignalDrawState,
        SignalAspects,
        SignalAspect,

        DrawLights,
        DrawLight,
        SemaphoreInfo,
        SemaphorePos,

        DirLights,
        DirShape,
        SignalDirDisplay,
        SignalNumClearAhead,

        SigSubType,
        SigSubSType,
        SigSubUnitSeq,
        SigFlashDuration,
        SigSubJnLinkIf,

        ScriptFiles,
        ScriptFile,

        SpeedMPH,
        SpeedKPH,

        // Signal object
        SignalUnits,
        SignalUnit,


        Class,
        Align,
        Depth,
        TdFile,
        Root,
        Elevation,
        TunnelEntrance,

        PointList,
        Velocity,

        // Speed Limit tokens
        NumSpeedLimit,
        SpeedLimit,
        Speed,

        Population,
        Area,
        Deployment,
        ScaleRange,
        StartPosition,
        EndPosition,
        StartType,
        EndType,
        Wire,

        // Train Controller Tokens
        Steam,
        Diesel,
        Electric,

        SafetyValvePressureDifference,
        Regulator,
        Cutoff,
        Brake_Engine,
        Brake_Train,
        Brake_Hand,
        Brake_Dynamic,
        Blower,
        HeatingTap,
        DampersFront,
        DampersBack,
        Firedoor,
        Injector1Steam,
        Injector1Water,
        Injector2Steam,
        Injector2Water,
        Sanding,
        Whistle,
        Throttle,
        DirControl,
        AWS,
        Horn,
        Vigilance,
        EngineStart,
        EngineStop,
        OverloadReset,
        PantographToggle1,
        PantographToggle2,
        CylinderCocks,
        WaterScoop,
        Mirrors,
        Wipers,
        Headlights,
        Taillights,
        Shovel,
        SmallEjectorOrCompressor,
        Simple_Control,
        Combined_Control,
        EmergencyStopResetToggle,
        EmergencyStopToggle,
        BellToggle,
        Gear,
        HighLowAcceleration,
        EngineBrakingButton,
        BailOffButton,
        NumNotches,
        Notch,
        MaxInHiAcceleration,
        MaxInLowAcceleration,
        MaxSandingTime,
        DoesHornTriggerBell,
        Dummy,

        DieselEngineType,

        GearBoxNumberOfGears,
        GearBoxDirectDriveGear,
        GearBoxMaxSpeedForGears,
        GearBoxMaxTractiveForceForGears,
        GearBoxOverspeedPercentageForFailure,
        GearBoxBackLoadForce,
        GearBoxCoastingForce,
        GearBoxUpGearProportion,
        GearBoxDownGearProportion,
        GearBoxOperation,
        GearBoxEngineBraking,
        MaxWheelAcceleration,

        StartDirection,
        EndDirection,
        ViewDbSphere,
        Radius,
        NoDirLight,
        Shadow,
        VDbId,
        VDbIdCount,
        AverageObjectPositionY,

        // TrainDB tokens
        TorsionModule,
        Centre,
        Angle,
        Type,
        None,
        Stiffness,
        Damping,
        rMin,
        rMax,
        MaxTilt,
        InertiaTensor,
        Box,
        Matrix3x3,
        Size,
        CentreOfGravity,
        Mass,
        Spring,
        Coupling,
        Hydraulic,
        Automatic,
        Chain,
        Bar,
        CouplingHasRigidConnection,
        CouplingUniqueType,
        Buffers,
        Bogie,
        Freight,
        Carriage,
        Draisine,
        Tender,
        WagonShape,
        WheelRadius,
        Wagons,
        MaxCurrent,
        MaxPower,
        MaxForce,
        MaxContinuousForce,
        r0,
        DerailRailHeight,
        DerailBufferForce,
        DerailRailForce,
        BoilerVolume,
        BoilerResponsiveness,
        BoilerEffectivity,
        CoalBurnage,
        MaxBoilerOutput,
        CylinderVolume,
        CylinderEffectivity,
        InjectorSizes,
        InjectorTypes,
        Injector1FeedRateMultiplierExhaust,
        Injector1FeedRateMultiplierLive,
        Injector2FeedRateMultiplierExhaust,
        Injector2FeedRateMultiplierLive,

        Injector1WaterValveVariance,
        Injector2WaterValveVariance,

        ExhaustLimit,
        BlastExponent,
        MaxBoilerPressure,
        MaxFireMass,
        MaxWaterMass,
        SafetyValvesSteamUsage,
        CylinderStroke,
        CylinderDiameter,
        NumCylinders,
        NumDampers,
        PrimingEffect,
        PrimingFactor,
        ShovelCoalMass,
        SmokeCombustion,
        NumWheels,
        Relaxation,
        MaxVelocity,
        MaxSteamHeatingPressure,

        IdealFireMass,
        BasicSteamUsage,
        BasicCoalUsage,
        MaxTenderCoalMass,
        MaxTenderWaterMass,
        MaxOilPressure,
        MaxTemperature,
        InjectorLimits1,
        InjectorLimits2,
        CylinderCocksPowerEfficiency,
        BoilerLength,
        BoilerRadius,
        DraftingEffect,
        PantographRaiseTime,
        Break,
        Superheater,
        BrakeVacuum,
        CriticalBlowbackLevel,
        Adheasion,
        ExtraParameters,
        Friction,
        MaxDieselLevel,
        DieselUsedPerHourAtMaxPower,
        DieselUsedPerHourAtIdle,
        SteamSmokeUnitsPerPoundOfFuel,
        SteamMaxSmokeUnitsReleaseRate,
        SteamGaugeGlassHeight,
        SteamMaxWaterHeightMultiple,
        SteamMinPrimingLevel,

        SteamPrimingPowerLossProportion,
        SteamPrimingWaterUsageIncreaseProportion,
        SteamPrimingInjectorsFailAtLevel,
        SteamPrimingReductionWithCylinderCocksOpen,

        // Diesel engine rpm parameters.
        DieselEngineMaxRPM,
        DieselEngineIdleRPM,
        DieselEngineStallRPM,
        DieselEngineSpeedOfMaxTractiveEffort,
        DieselEngineMaxRPMChangeRate,

        DoesBrakeCutPower,
        BrakeCutsPowerAtBrakeCylinderPressure,

        // Diesel engine smoke effect parameters.
        DieselSmokeEffectMaxMagnitude,
        DieselSmokeEffectInitialMagnitude,
        DieselSmokeEffectInitialSmokeRate,
        DieselSmokeEffectMaxSmokeRate,

        // Load the brake system parameters for a wagons and engines.
        BrakeEquipmentType,
        BrakeSystemType,
        MaxBrakeForce,
        MaxHandbrakeForce,
        NumberOfHandbrakeLeverSteps,
        QuickReleaseTriggerRate,
        QuickReleaseMagnification,
        QuickApplicationTriggerRate,
        QuickApplicationMagnification,
        EmergencyBrakeResMaxPressure,
        EmergencyBrakeTriggerRate,
        TrainPipeLeakRate,
        AuxilaryLeakRate,
        TripleValveRatio,
        MaxReleaseRate,
        MaxApplicationRate,
        MaxAuxilaryChargingRate,
        EmergencyResCapacity,
        EmergencyResChargingRate,
        RetainingValveSlowReleaseRate,
        BrakeCylinderPressureForMaxBrakeBrakeForce,
        EmergencyResVolumeMultiplier,

        BrakeDistributorFullServicePressure,
        BrakeDistributorEmergencyApplicationPressure,
        BrakeDistributorMaxAuxilaryResPressure,
        BrakeDistributorEmergencyResPressure,
        BrakeDistributorReleaseRate,
        BrakeDistributorApplicationRate,
        BrakeDistributorEmergencyApplicationRate,
        BrakeDistributorTrainPipeAuxResPressureDifferenceForApplication,
        BrakeDistributorTrainPipeControlRatio,
        BrakeDistributorNormalFullReleasePressure,

        MaxBrakeCylinderPressure,
        BrakeAirPressureGainInResForOnePoundFromPipe,
        BrakeAuxilaryResPressure,
        BrakeEmergencyResPressure,

        // Load brake system parameters for engine and train support.
        AirBrakeMaxMainResPipePressure,
        AirBrakesAirCompressorPowerRating,
        AirBrakesMainMinResAirPressure,
        AirBrakesMainMaxAirPressure,
        AirBrakesMainResVolume,
        AirBrakesAirCompressorWattage,
        AirBrakesCompressorRestartPressure,
        AirBrakesSteamUsageRate,
        AirBrakesAirUsedPerPoundsOfBrakePipePressure,
        AirBrakesHasLowPressureTest,
        AirBrakesIsCompressorElectricOrMechanical,

        VacuumBrakesExhauster1PowerRating,
        VacuumBrakesExhauster2PowerRating,
        VacuumBrakesExhausterIdlingPowerRating,
        VacuumBrakesIsExhausterElectricOrMechanical,
        VacuumBrakesExhauster1Wattage,
        VacuumBrakesExhauster2Wattage,

        VacuumBrakeBoilerPressureForZeroVacuum,
        VacuumBrakesSmallEjectorPowerRating,
        VacuumBrakesMinBoilerPressureMaxVacuum,
        VacuumBrakesSmallEjectorUsageRate,
        VacuumBrakesLargeEjectorUsageRate,
        VacuumBrakesHasVacuumPump,
        VacuumBrakesVacuumPumpPower,
        VacuumBrakesVacuumPumpResistance,

        DynamicBrakesMinUsableSpeed,
        DynamicBrakesMaximumEffectiveSpeed,
        DynamicBrakesMaximumForce,
        DynamicBrakesResistorCurrentLimit,
        DynamicBrakesCutInSpeed,
        DynamicBrakesMaxAirBrakePressure,
        DynamicBrakesFadingSpeed,
        DynamicBrakesDelayTimeBeforeEngaging,
        DynamicBrakesMaximumSpeedForFadeOut,
        DynamicBrakesEffectAtMaximumFadeOut,
        DynamicBrakesHigherSpeedCurveExponent,
        DynamicBrakesLowerSpeedCurveExponent,
        DynamicBrakesNumberOfControllerNotches,
        DynamicBrakeHasAutoBailOff,

        TrainBrakesControllerDirectControlExponent,
        TrainBrakesControllerFullQuickReleaseStart,
        TrainBrakesControllerGraduatedQuickReleaseStart,
        TrainBrakesControllerReleaseStart,
        TrainBrakesControllerReleaseLappedStart,
        TrainBrakesControllerHoldLappedStart,
        TrainBrakesControllerHoldStart,
        TrainBrakesControllerEPApplyStart,
        TrainBrakesControllerEPHoldStart,
        TrainBrakesControllerRunningStart,
        TrainBrakesControllerSelfLapStart,
        TrainBrakesControllerGraduatedSelfLapStart,
        TrainBrakesControllerGraduatedSelfLapLimitedStart,
        TrainBrakesControllerGraduatedSelfLapLimitedHoldingStart,
        EngineBrakesControllerGraduatedSelfLapLimitedHoldingStart,
        TrainBrakesControllerGraduatedSelfLapLimitedKeepPsiStart,

        TrainBrakesControllerNeutralhandleOffStart,
        TrainBrakesControllerMinimalReductionStart,
        TrainBrakesControllerApplyStart,

        TrainBrakesControllerFullServiceStart,
        TrainBrakesControllerSuppressionStart,
        TrainBrakesControllerContinuousServiceStart,
        TrainBrakesControllerEmergencyStart,
        TrainBrakesControllerMaxApplicationRate,
        TrainBrakesControllerMaxQuickReleaseRate,
        TrainBrakesControllerMaxReleaseRate,
        TrainBrakesControllerEmergencyApplicationRate,
        TrainBrakesControllerMinSystemPressure,
        TrainBrakesControllerMaxSystemPressure,
        TrainBrakesControllerEmergencyBrakeTimePenalty,
        TrainBrakesControllerMinPressureReduction,
        TrainBrakesControllerFullServicePressureDrop,
        TrainBrakesControllerMaxPressureDropInNormalApplication,

        TrainBrakesControllerMinContinuousService,
        TrainBrakesControllerMaxContinuousService,

        EngineBrakesControllerDirectControlExponent,
        EngineBrakesControllerFullQuickReleaseStart,
        EngineBrakesControllerGraduatedQuickReleaseStart,
        EngineBrakesControllerReleaseStart,
        EngineBrakesControllerReleaseLappedStart,
        EngineBrakesControllerHoldLappedStart,
        EngineBrakesControllerHoldStart,
        EngineBrakesControllerEPApplyStart,
        EngineBrakesControllerEPHoldStart,
        EngineBrakesControllerRunningStart,
        EngineBrakesControllerSelfLapStart,
        EngineBrakesControllerGraduatedSelfLapStart,
        EngineBrakesControllerGraduatedSelfLapLimitedStart,
        EngineBrakesControllerGraduatedSelfLapLimitedKeepPsiStart,

        EngineBrakesControllerNeutralHandleOffStart,
        EngineBrakesControllerMinimalReductionStart,
        EngineBrakesControllerApplyStart,
        EngineBrakesControllerFullServiceStart,
        EngineBrakesControllerSuppressionStart,
        EngineBrakesControllerContinuousServiceStart,
        EngineBrakesControllerEmergencyStart,
        EngineBrakesControllerMaxApplicationRate,
        EngineBrakesControllerMaxQuickReleaseRate,
        EngineBrakesControllerMaxReleaseRate,
        EngineBrakesControllerEmergencyApplicationRate,
        EngineBrakesControllerMinSystemPressure,
        EngineBrakesControllerMaxSystemPressure,
        EngineBrakesControllerEmergencyBrakeTimePenalty,
        EngineBrakesControllerMinPressureReduction,
        EngineBrakesControllerFullServicePressureDrop,
        EngineBrakesControllerMaxPressureDropInNormalApplication,

        EngineBrakesControllerHasProportionalBrake,
        EngineBrakesProportionalBrakeLag,
        EngineBrakesProportionalBrakeOutputReleasedPressure,
        EngineBrakesProportionalBrakeOutputAppliedPressure,
        EngineBrakesProportionalBrakeInputReleasedPressure,
        EngineBrakesProportionalBrakeInputAppliedPressure,
        BrakesEngineBrakeType,
        BrakesTrainBrakeType,

        EngineBrakesControllerMinContinuousService,
        EngineBrakesControllerMaxContinuousService,

        // engine configurable controllers setup
        EngineControllers,
        BrakesEngineControllers,

        // Regulator parameters.
        RegulatorValveType,
        RegulatorSecondValveStartPosition,
        RegulatorPilotValveExponent,
        RegulatorSecondValveExponent,
        RegulatorPilotValveFullOpenning,
        RegulatorMainValveInitialOpenning,

        // Blower parameters.
        SteamBlowerEffectExponent,
        SteamBlowerEfficiencyExponent,
        SteamBlowerMaxSteamUsageRate,
        SteamBlowerMaxProportionOfBlastEffect,

        // Water scoop parameters.
        SteamWaterScoopMinPickupSpeed,
        SteamWaterScoopMaxPickupSpeed,
        SteamWaterScoopResistance,
        SteamWaterScoopMaxPickupRate,

        // Fireman's parameters.
        SteamFiremanConditionLossRateConstant,
        SteamFiremanConditionGainRateConstant,
        SteamFiremanEnergyLossRateConstant,
        SteamFiremanEnergyGainRateConstant,
        SteamFiremanMaxConstantFiringRate,
        SteamFiremanMaxPossibleFiringRate,
        SteamFiremanEnergyUsedPerPoundOfCoal,
        SteamFiremanIsMechanicalStoker,

        // Oil burner parameters.
        SteamOilBurnerMinBurnRate,
        SteamOilBurnerMaxBurnRate,
        SteamOilBurnerControlExponent,
        SteamOilBurnerSteamFromOnePoundOfOilBurnt,

        SteamOilBurnerMaxRateOfTempChange,
        SteamOilBurnerEfficiencyExponent,
        SteamOilBurnerMinBurnerTemperature,
        SteamOilBurnerMaxBurnerTemperature,

        SteamOilBurnerTemperatureChnageExponent,
        SteamOilBurnerSteamGenerationExponent,
        SteamOilBurnerSmokeForOnePoundOfOil,
        SteamOilBurnerSteamFuelType,

        // Cylinder cocks operation.
        SteamCylinderCocksOperation,

        CutoffMaxReverse,
        CutoffMaxForward,
        SteamSmallestCutoff,
        NumberOfDamperSteps,

        // Does the engine need a tender for coal and water?
        IsTenderRequired,

        Handbrake,
        AntiSlip,
        Inside,
        PassengerCabinFile,
        PassengerCabinHeadPos,
        Thumbnail,
        HeadOut,
        Mirror,
        Wiper,
        Door,
        Pantograph,
        Wheelset,
        IntakePoint,
        FreightGrain,
        FreightCoal,
        FreightGravel,
        FreightSand,
        FuelWater,
        FuelCoal,
        FuelDiesel,
        SpecialMail,
        FreightAnim,
        EngineOperatingProcedures,


        // Track Item Table entries
        TrItemRefs,
        TrItemRef,
        TrItemId,
        TrItemSData,
        TrItemPData,
        TrItemRData,
        TrItemSRData,
        TrItemTable,
        EmptyItem,
        SignalItem,
        PickupItem,
        PlatformItem,
        HazzardItem,
        CarSpawnerItem,
        SidingItem,
        LevelCrItem,
        SpeedPostItem,
        SoundRegionItem,
        CrossoverItem,

        // Trains in WDB
        TrainWagons,
        WagonType,
        CoupledWith,
        WagonData,
        EngineData,
        FirstWagon,
        QDirection,
        EngineVariables,
        TrainData,
        Inverse,
        Flip,
        NotCoupledToActiveTrain,
        NextWagonUID,
        Durability,


        // train sfx data
        Effects,
        FrontView,
        LeftView,
        RightView,
        HornFX,
        BellFX,
        AlerterFX,
        SteamSpecialEffects,
        CylindersFX,
        DrainpipeFX,
        StackFX,
        SafetyValvesFX,
        WhistleFX,

        DieselSpecialEffects,
        Exhaust1,
        Exhaust2,

        // Used for markers
        Marker,
        Coords,
        mapmm,

        TrItemSignal,
        TrItemDest,
        TrDestName,
        TrDestNameId,

        NumSections,
        MainRoute,
        NumPaths,
        ClearanceDist,

        // cameras
        Camera,
        CamType,
        CamTypeBrick,
        CamTypeFrontTracking,
        CamTypeRearTracking,
        CamTypePassenger,
        CamTypeCab,
        CamTypeSpotter,
        CamTypeDerail,
        CamTypeHeadOut,
        CamTypeCoupling,
        CamTypeYard,

        CamControlNo,
        CamControlRotate,
        CamControlFull,
        CamControlStrafe,
        CamControlIsometric,
        CamControlViewSwitch,
        CamControlTethered,
        CamControlDerail,
        CamControlCoupling,
        CamControlYard,

        WagonNum,
        RotationLimit,
        Fov,
        ZClip,
        CameraOffset,
        ObjectOffset,

        Subpath,
        FileIndex,

        TrGUI_Editbox_Vars,
        TrGUI_Editbox_Focus_Lock,
        TrGUI_Editbox_Alphanumeric,

        TrGUI_Listbox_Node,
        TrGUI_List_Slots,
        TrGUI_Listbox_SlotHt,
        TrGUI_Listbox_Max,
        TrGUI_Listbox_Min_Slots,
        TrGUI_Listbox_Slider,

        TrGUI_Scroll_Delay,

        TrGUI_Region_Name,
        TrGUI_Region_Surf,
        TrGUI_Region_Disabled,
        TrGUI_Region_Dis_Surf,
        TrGUI_Region_Area,
        TrGUI_Region_Cursor,
        TrGUI_Region_Parent,
        TrGUI_Region_Layer,
        TrGUI_Region_Drag_Target,
        TrGUI_Region_Font,
        TrGUI_Region_DisFont,
        TrGUI_Region_Shadow_Font,
        TrGUI_Region_TextArea,
        TrGUI_Region_Text_Owned,

        TrGUI_Slider_Vertical,
        TrGUI_Slider_Centre,
        TrGUI_Slider_Min_Max,
        TrGUI_Slider_Quantized,
        TrGUI_Slider_Back,
        TrGUI_Slider_Area,

        TrGUI_Tooltip_Text,
        TrGUI_Drag_Bitmap,
        TrGUI_Text_Pos,
        TrGUI_Mouse_LDn,
        TrGUI_Mouse_RDn,
        TrGUI_Mouse_LDblClick,
        TrGUI_Mouse_RDblClick,
        TrGUI_Mouse_LUp,
        TrGUI_Mouse_RUp,
        TrGUI_Mouse_LHeld,
        TrGUI_Mouse_RHeld,
        TrGUI_Mouse_Focus,
        TrGUI_Hotspot,
        TrGUI_Cursor_Surfs,
        TrGUI_Widget,
        TrGUI_Icon_Set,

        TrGUI_Next_Screen,
        TrGUI_Parent_Screen,
        TrGUI_Loading_Screen,
        TrGUI_Textbox_Slider,
        TrGUI_IOM_Filename,

        TrGUI_Debrief_Screen,

        TrGUI_Up_Button,
        TrGUI_Dn_Button,
        TrGUI_Num_Lines,

        TrGUI_Onoff_Dn_Surf,
        TrGUI_Onoff_Radio,

        TrGUI_Onoff_Popup_Centre,
        TrGUI_Timeout,
        TrGUI_Popup_Non_Modal,

        TrGUI_Button_Down_Surf,
        TrGUI_Button_Down_Font,
        TrGUI_Button_Group,
        TrGUI_Group_Max_On,
        TrGUI_State3_Hi_Font,
        TrGUI_State3_Hi_Surf,

        TrGUI_ComboButton_Popup,
        TrGUI_ComboButton_Listbox,
        TrGUI_ComboButton_Display_Region,
        TrGUI_ComboButton_Open_Up,
        TrGUI_ComboButton_Use_Bitmaps,
        TrGUI_ComboButton_No_Text,
        TrGUI_ComboButton_Listbox_Sound,

        TrGUI_Overlay_Type,

        TrGUI_Anim_Frame_Area,
        TrGUI_Anim_Num_Frames,
        TrGUI_Anim_Delay,
        TrGUI_Anim_Reverse,
        TrGUI_Anim_Loop,

        // Telepole Data
        TPoleConfigData,
        TPoleConfig,
        Config,
        Separation,

        //extended shape stuff
        ESD_Bounding_Box,
        ESD_Tunnel,
        ESD_Complex_Box,
        ESD_Complex,
        ESD_Sphere,
        ESD_Alternative_Texture,
        ESD_Shape_Count,
        ESD_Detail_Level,
        ESD_No_Visual_Obstruction,
        ESD_Snapable,
        ESD_Software_Dlev,
        ESD_SubObj,
        ESD_Headlight,

        // Platform tokens
        PlatformName,
        PlatformData,
        PlatformTrItemData,
        PlatformMinWaitingTime,
        PlatformNumPassengersWaiting,

        // Pickup tokens
        SpeedRange,
        PickupType,
        PickupAnimData,
        PickupCapacity,
        PickupTrItemData,

        // CarSpawner tokens
        CarFrequency,
        CarAvSpeed,
        CarSpawnerTrItemData,

        // Siding tokens
        SidingName,
        SidingData,
        SidingTrItemData,

        // Level Crossing tokens
        LevelCrParameters,
        LevelCrData,
        LevelCrTiming,
        LevelCrTrItemData,

        // Speedpost tokens
        SpeedpostFlags,
        SpeedpostNumber,
        SpeedpostTrItemData,

        Speedpost_Set_Count,
        Speedpost_Set,
        Speed_Sign_Shape,
        Speed_Warning_Sign_Shape,
        Speed_Resume_Sign_Shape,
        Speed_Digit_Tex,
        Milepost_Shape,
        Milepost_Digit_Tex,
        Restricted_Shape,
        End_Restricted_Shape,
        Speed_Text_Size,
        Milepost_Text_Size,

        // Crossovers tokens
        CrossoverTrItemData,

        // TrackType Tokens
        TrackType,

        // Gantry tokens
        Style,
        GantrySets,
        GantrySet,
        GantryTable,
        GantryTableEntry,
        Distance,

        // gantry stuff used by trsignal
        GantryInfo,
        GantryShape,
        GantrySize,

        // Forest tokens
        Width,
        Height,
        TreeTexture,
        TreeSize,
        Birds,

        //Track hazard stuff
        Idle_Key,
        Idle_Key2,
        Surprise_Key_Left,
        Surprise_Key_Right,
        Success_Scarper_Key,
        Fail_Scarper_Key,

        // Soundregion tokens
        SoundregionTrackType,
        SoundregionRoty,

        // Player data
        Players,
        Player,

        // CabView Data
        CabView,
        CabViewType,
        CabViewFile,
        CabViewWindow,
        CabViewWindowFile,
        CabViewControls,
        Gauge,
        Dial,
        Digital,
        DigitalClock,
        AnalogueClock,
        MultiStateDisplay,
        CabSignalDisplay,
        TwoState,
        TriState,
        CombinedControl,
        Lever,
        Firebox,
        ZeroPos,
        PositiveColour,
        NegativeColour,
        DecreaseColour,
        ControlColour,
        Orientation,
        Units,
        Pivot,
        ScalePos,
        Accuracy,
        AccuracySwitch,
        LeadingZeros,
        Justification,
        States,
        State,
        SwitchVal,
        MouseControl,
        NumFrames,
        NumPositions,
        NumValues,
        DirIncrease,


        // Route File Data
        RouteID,
        Name,
        Description,
        Electrified,
        Mountains,
        OverheadWireHeight,
        PassengerRuleSet,
        FreightRuleSet,
        SignalSet,
        TrackGauge,
        Era,
        Environment,
        SpringClear,
        SpringRain,
        SpringSnow,
        SummerClear,
        SummerRain,
        SummerSnow,
        AutumnClear,
        AutumnRain,
        AutumnSnow,
        WinterClear,
        WinterRain,
        WinterSnow,
        TerrainErrorScale,
        RouteStart,
        LoadingScreen,
        MilepostUnitsMiles,
        MilepostUnitsKilometers,
        MaxLineVoltage,
        DefaultSignalSMS,
        DefaultCrossingSMS,
        DefaultWaterTowerSMS,
        DefaultCoalTowerSMS,
        DefaultDieselTowerSMS,
        TempRestrictedSpeed,

        // Activity File Data
        PathID,
        Mode,
        Briefing,
        Player_Train_Config,
        CoalLevel,
        WaterLevel,
        FuelLevel,
        StartingSpeed,
        EndingSpeed,
        Season,
        Weather,
        StartTime,
        Hazards,
        Animals,
        Workers,
        Debris,
        Voltage,
        Difficulty,
        Severity,
        StationProperties,
        Station,
        ID,
        Value,
        FailedSignals,
        FailedSignal,
        FreightPoints,
        FreightPoint,
        Action,
        MailPoints,
        MailPoint,
        ActivityObjects,
        ActivityObject,
        ActivityRestrictedSpeedZones,
        ActivityRestrictedSpeedZone,
        ActivityFailedSignals,
        ActivityFailedSignal,
        Tile,
        ObjectType,
        Events,
        Location,
        Trigger,
        Outcomes,
        PickupIdAndAmount,
        TextToDisplayOnCompletionIfTriggered,
        TextToDisplayOnCompletionIfNotTriggered,
        TextToDisplayDescriptionOfTask,
        BreakSpeedRecord,
        CoupleToTrainset,
        LeaveWagons,
        PickedUpPassengers,
        PickedUpCargo,
        ReachedSpeed,
        Time,
        AITrains,
        Traffic_Definition,
        Service_Definition,
        Train_Config,
        Reversable_Event,

        ActivitySuccessText,
        ActivityFailText,

        TriggerOnStop,

        CompleteActivity,

        IncActLevel,
        DecActLevel,
        ActivateEvent,
        RestoreActLevel,
        DisplayMessage,
        PlaySound,
        ActivitySuccess,
        ActivityFail,
        StartIgnoringSpeedLimits,
        StopIgnoringSpeedLimits,

        Activation_Level,

        EventCategoryLocation,
        EventCategoryTime,
        EventCategoryAction,

        EventTypeLocation,
        EventTypeTime,

        //editor specific
        NextServiceUID,
        NextActivityObjectUID,

        //action events
        EventTypeAllStops,
        EventTypePickUpPassengers,
        EventTypePickUp,
        EventTypeReachSpeed,
        EventTypePickUpWagons,
        EventTypeDropOffWagonsAtLocation,
        EventTypeAssembleTrain,
        EventTypeAssembleTrainAtLocation,

        // action event data
        Wagon_List,

        // Timetable/Service Data
        TimeTable,
        StartInWorld,
        EndInWorld,
        StationStop,
        PlatformStartID,
        PlatformEndID,
        ArrivalTime,
        MinWaitingTime,
        DepartTime,
        DistanceDownPath,
        Efficiency,
        SkipCount,

        // TrackDB Paths
        TrackPDPs,
        TrackPDP,
        TrackPaths,
        TrackPath,
        TrPathName,
        TrPathStart,
        TrPathEnd,
        TrPathNodeSid,
        TrPathPDPRef,
        TrPathNodes,
        TrPathNode,
        TrPathFlags,

        // Sound
        ScalabiltyGroup,
        Streams,
        Stream,
        Triggers,
        Discrete_Trigger,
        Variable_Trigger,
        Initial_Trigger,
        Dist_Travelled_Trigger,
        Random_Trigger,
        Delay_Min_Max,
        Volume_Min_Max,
        Dist_Min_Max,
        Speed_Inc_Past,
        Speed_Dec_Past,
        Distance_Inc_Past,
        Distance_Dec_Past,
        Variable1_Inc_Past,
        Variable1_Dec_Past,
        Variable2_Inc_Past,
        Variable2_Dec_Past,
        Variable3_Inc_Past,
        Variable3_Dec_Past,
        PlayOneShot,
        FileList,
        File,
        SelectionMethod,
        RandomSelection,
        SequentialSelection,
        StartLoop,
        StartLoopRelease,
        ReleaseLoopRelease,
        ReleaseLoopReleaseWithJump,
        EnableTrigger,
        DisableTrigger,
        Volume,
        Stereo,
        LoadAllWaves,
        Priority,
        SetStreamVolume,
        Ignore3d,


        VolumeCurve,
        FrequencyCurve,
        SpeedControlled,
        DistanceControlled,
        Variable1Controlled,
        Variable2Controlled,
        Variable3Controlled,
        CurvePoints,
        Granularity,

        Activation,
        Deactivation,
        PassengerCam,
        CabCam,
        ExternalCam,
        //SID_DISTANCE
        //SID_TRACKTYPE

        SoundTimer,
        AudioStream,
        SoundSystem,
        TrainMinMaxDistance,
        AmbientMinMaxDistance,
        Rolloff,

        // Train monitoring device.
        MonitoringDeviceMonitorTimeLimit,
        MonitoringDeviceAlarmTimeLimit,
        MonitoringDevicePenaltyTimeLimit,
        MonitoringDeviceCriticalLevel,
        MonitoringDeviceResetLevel,
        MonitoringDeviceAppliesFullBrake,
        MonitoringDeviceAppliesEmergencyBrake,
        MonitoringDeviceAppliesCutsPower,
        MonitoringDeviceAppliesShutsDownEngine,
        MonitoringDeviceAlarmTimeBeforeOverSpeed,

        MonitoringDeviceResetOnZeroSpeed,
        MonitoringDeviceResetOnZeroThrottle,
        MonitoringDeviceResetOnDirectionNeutral,
        MonitoringDeviceResetOnEngineAtIdle,
        MonitoringDeviceResetOnBrakeOff,
        MonitoringDeviceResetOnBrakeFullyOn,
        MonitoringDeviceResetOnDynamicBrakeOff,
        MonitoringDeviceResetOnDynamicBrakeOn,
        MonitoringDeviceResetOnResetButton,

        MonitoringDeviceDoesSuppressionReset,
        MonitoringDeviceTriggerOnOverspeed,
        MonitoringDeviceTriggerOnHighCurrent,
        MonitoringDeviceTriggerOnLowMainResPressire,
        MonitoringDeviceTriggerOnOverRPM,
        MonitoringDeviceTriggerOnTrackOverspeed,
        MonitoringDeviceTriggerOnTrackOverspeedMargin,

        AWSMonitor,
        VigilanceMonitor,
        EmergencyStopMonitor,
        OverspeedMonitor,

        // Tokens used only by TsDataEx
        LCRegion,
        DEMRegion,
        Region,
        FileSampleSize,
        FileSamples,
        FileSampleBytes,
        BigEndian,
        SampleFiles,
        SampleFile,
        SFlags,

        // Tokens for lights

        Light,
        Conditions,
        Headlight,
        Unit,
        Penalty,
        Control,
        Service,
        TimeOfDay,
        ActiveCab,
        InactiveCab,
        FadeIn,
        FadeOut,
        Duration,
        LightColour,
        Azimuth,
        Transition,
        Cycle,

        // Save game tokens - these could all be re-used tokens really
        Serial,
        SaveHeader,
        SaveTDBs,
        SaveTDB,
        SaveRDB,
        SaveTDBNodes,
        SaveTDBNode,
        SaveTrItems,
        SaveTrItem,
        SaveVNodeOwners,
        SaveObj,

        SaveStatic,
        SaveCollobj,
        SavePhysobj,
        SaveWagon,
        SaveEngine,
        SaveTrain,

        SaveController,
        SaveActivity,
        SaveEvents,
        SaveEvent,
        SaveServices,
        SaveService,
        SaveActObjs,
        SaveActObjWagonList,
        SaveActObjSpeedMon,
        SaveWagonUIDs,
        SaveDebrief,

        SaveTitle,
        SaveSuccessfulStrings,
        SaveSuccessfulString,
        SaveUnSuccessfulStrings,
        SaveUnSuccessfulString,
        SaveTimetableEntries,
        SaveTimetableEntry,
        SaveWorkOrderEntries,
        SaveWorkOrderEntry,
        SaveWorkOrderType,
        SaveWorkOrderTaskText,
        SaveWorkOrderWagonIds,
        SaveWorkOrderLocationText,
        SaveWorkOrderNotesText,
        SaveWorkOrderCompleted,

        SaveSStops,
        SaveSStop,
        SaveTrFollower,
        SaveAIWagons,
        SaveAIWagon,
        SaveSPUnavail,
        SaveSP,

        // version checking tokens I think we might need
        Version_Route,
        Version_TrainDB,
        Version_Consist,
        Version_Path,
        Version_Traffic,
        Version_Service,

        // player service and traffic definitions
        Player_Traffic_Definition,
        Player_Service_Definition,
        // encapsulation for activity files
        Tr_Activity,

        //level crossing crash probability
        CrashProbability,

        //trackdb
        TrackDB,

        // passenger capacity of a wagon
        PassengerCapacity,

        // min time it takes to run the engine up to its max force
        RunUpTimeToMaxForce,
        VolumeTweak,
        GravityScale,
        DerailScale,
        TimetableTollerance,

        DEMPath,

        // these assigned ID's are arbitrary - I haven't seen them in a compressed MSTS W file yet
        // TODO determine proper ID from a compressed world file
        CarSpawner, 
        Siding, 
        Dyntrack,
        Transfer,
        Gantry,
        Pickup,  

        Wagon,
        Engine
    }

}
